@@ -6,6 +6,7 @@ from datetime import datetime
 from typing import Dict, List, Optional, Set, Tuple, Union
 
 from asn1crypto import cms, crl, x509
+from asn1crypto.crl import CRLEntryExtensionId
 from cryptography.exceptions import InvalidSignature
 
 from pyhanko_certvalidator._state import ValProcState
@@ -1382,31 +1383,37 @@ def find_cert_in_list(
     ]
 
     if isinstance(cert, x509.Certificate):
-        cert_serial = cert.serial_number
+        cert_serial = cert['tbs_certificate']['serial_number'].dump()
     else:
-        cert_serial = cert['ac_info']['serial_number'].native
+        cert_serial = cert['ac_info']['serial_number'].dump()
 
     last_issuer_name = crl_authority_name
+
+    cert_issuer_extension_id = CRLEntryExtensionId('certificate_issuer').dump()
+
     for revoked_cert in revoked_certificates:
-        # If any unknown critical extensions, the entry can not be used
-        if revoked_cert.critical_extensions - KNOWN_CRL_ENTRY_EXTENSIONS:
-            raise NotImplementedError()
-
-        if (
-            revoked_cert.issuer_name
-            and revoked_cert.issuer_name != last_issuer_name
-        ):
-            last_issuer_name = revoked_cert.issuer_name
-        if last_issuer_name != cert_issuer_name:
+        # This looks like a hack, but we have to look up the certificate_issuer
+        # extension for every entry, since its value remains in effect for
+        # future entries as well! (and PKITS has a test case for that...)
+        # Since parsing those extensions every time is expensive for large CRLs,
+        # we guard it with a dumb heuristic check: does the binary encoding
+        # of that extension's OID appear anywhere in the entry's payload?
+        # If not, we move on. If it does appear, we parse the extensions.
+        if cert_issuer_extension_id in revoked_cert.dump():
+            if revoked_cert.issuer_name:
+                last_issuer_name = revoked_cert.issuer_name
+        if revoked_cert['user_certificate'].dump() != cert_serial:
             continue
-
-        if revoked_cert['user_certificate'].native != cert_serial:
+        if last_issuer_name != cert_issuer_name:
             continue
 
         if not revoked_cert.crl_reason_value:
             crl_reason = crl.CRLReason('unspecified')
         else:
             crl_reason = revoked_cert.crl_reason_value
+        # If any unknown critical extensions, the entry can not be used
+        if revoked_cert.critical_extensions - KNOWN_CRL_ENTRY_EXTENSIONS:
+            raise NotImplementedError()
 
         return revoked_cert['revocation_date'].native, crl_reason
 
